# Generated by roxygen2: do not edit by hand

S3method(cor,default)
S3method(cor,symbolic_interval)
S3method(cor,symbolic_tbl)
S3method(cov,default)
S3method(cov,symbolic_interval)
S3method(cov,symbolic_tbl)
S3method(print,ggInterval_hist_result)
S3method(scale,default)
S3method(scale,symbolic_interval)
S3method(scale,symbolic_tbl)
S3method(sd,default)
S3method(sd,symbolic_interval)
S3method(sd,symbolic_tbl)
S3method(summary,default)
S3method(summary,symbolic_interval)
S3method(summary,symbolic_modal)
S3method(summary,symbolic_tbl)
export(RSDA2sym)
export(classic2sym)
export(cor)
export(cov)
export(ggInterval)
export(ggInterval_2Dhist)
export(ggInterval_2DhistMatrix)
export(ggInterval_3Dscatterplot)
export(ggInterval_CRplot)
export(ggInterval_MMplot)
export(ggInterval_PCA)
export(ggInterval_boxplot)
export(ggInterval_corrplot)
export(ggInterval_hist)
export(ggInterval_indexImage)
export(ggInterval_indexplot)
export(ggInterval_lineplot)
export(ggInterval_radarplot)
export(ggInterval_scatterMatrix)
export(ggInterval_scatterplot)
export(ggInterval_tsplot)
export(scale)
export(sd)
export(summary)
import(R6)
import(ggforce)
import(ggplot2)
import(ggthemes)
import(rlang)
import(stats)
import(tidyverse)
import(vctrs)
importFrom(RSDA,classic.to.sym)
importFrom(RSDA,is.sym.interval)
importFrom(dplyr,arrange)
importFrom(ggpubr,ggarrange)
importFrom(grDevices,gray.colors)
importFrom(grDevices,grey.colors)
importFrom(gridExtra,marrangeGrob)
importFrom(gtools,odd)
importFrom(magrittr,"%>%")
importFrom(prodlim,row.match)
importFrom(stringr,str_split)
importFrom(tibble,as_tibble)
importFrom(tibble,column_to_rownames)
importFrom(tibble,rownames_to_column)
importFrom(tidyr,separate)
