% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggInterval_3Dscatterplot.R
\name{ggInterval_3Dscatterplot}
\alias{ggInterval_3Dscatterplot}
\title{3D scatter plot for interval data}
\usage{
ggInterval_3Dscatterplot(data = NULL,mapping = aes(NULL),scale=FALSE)
}
\arguments{
\item{data}{A ggInterval object. It can also be either an RSDA object or
a classical data frame, which will be automatically converted to ggInterval
data.}

\item{mapping}{Set of aesthetic mappings created by aes() or aes_().
If specified and inherit. aes = TRUE (the default),
it is combined with the default mapping at the top level of
the plot. You must supply mapping if there is no plot mapping.
It is the same as the mapping of ggplot2.}

\item{scale}{A boolean variable. \code{TRUE} standardizes the data and
\code{FALSE} leaves the data unchanged.
If variance is too large(or small) or the difference between
two variables are too large,it will be distorted or difficult to see, which
may happen when variables are measured in different units.}
}
\value{
Return a ggplot2 object (It will still be 2-Dimension).
}
\description{
Visualize the three continuous variable distribution
by collecting all vertices in each interval to form a shape of
cube.Also show the difference between each group.
}
\examples{
ggInterval_3Dscatterplot(facedata[1:5, ], aes(x = BC, y = EH, z = GH))

}
