% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggInterval_PCA.R
\name{ggInterval_PCA}
\alias{ggInterval_PCA}
\title{Vertices PCA for interval data}
\usage{
ggInterval_PCA(data = NULL,mapping = aes(NULL),plot=TRUE,
                      concepts_group=NULL, poly = FALSE, adjust = TRUE,
                      showLabels = TRUE, labelSize = 3,
                      checkOverlap = FALSE)
}
\arguments{
\item{data}{A ggInterval object. It can also be either RSDA object or
classical data frame, which will be automatically convert to ggInterval
data.}

\item{mapping}{Set of aesthetic mappings created by aes() or aes_().
If specified and inherit. aes = TRUE (the default),
it is combined with the default mapping at the top level of
the plot. You must supply mapping if there is no plot mapping.
It is the same as the mapping of ggplot2.}

\item{plot}{Boolean variable,Auto plot (if TRUE).It can also plot by
its inner object}

\item{concepts_group}{color with each group of concept}

\item{poly}{if plot a poly result}

\item{adjust}{adjust sign of the principal component}

\item{showLabels}{Logical. If \code{TRUE}, label interval scores in the plot.}

\item{labelSize}{Numeric text size used when \code{showLabels = TRUE}.}

\item{checkOverlap}{Logical. Passed to \code{geom_text()} to suppress
overlapping labels when needed.}
}
\value{
A ggplot object for PC1, PC2, and interval scores with related outputs.
\itemize{
  \item scores_interval - The interval scores after PCA.
  \item ggplotPCA - a ggplot object with x-axis and y-axis are PC1 and
  PC2.
  \item others - others are the returns values of princomp.
}
}
\description{
ggInterval_PCA performs a principal components
analysis on the given numeric interval data and returns the
results of princomp, a ggplot object, and interval scores.
}
\examples{
\donttest{
Subjects <- substr(rownames(facedata), 1, 3)
p <- ggInterval_PCA(facedata, plot = FALSE, concepts_group = Subjects)
p$ggplotPCA
p$scores_interval
ggInterval_PCA(facedata, poly = TRUE, concepts_group = Subjects)
}

}
