% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utilities.R
\name{cor}
\alias{cor}
\alias{cor.default}
\alias{cor.symbolic_tbl}
\alias{cor.symbolic_interval}
\title{Generic function for the correlation}
\usage{
cor(x, ...)

\method{cor}{default}(
  x,
  y = NULL,
  use = "everything",
  method = c("pearson", "kendall", "spearman"),
  ...
)

\method{cor}{symbolic_tbl}(x, ...)

\method{cor}{symbolic_interval}(x, y, method = c("centers", "B", "BD", "BG"), ...)
}
\arguments{
\item{x}{First symbolic variables.}

\item{...}{As in R cor function.}

\item{y}{Second symbolic variables.}

\item{use}{an optional character string giving a method for computing
correlation in the presence of missing values. This must be (an abbreviation of)
 one of the strings 'everything', 'all.obs', 'complete.obs', 'na.or.complete',
 or 'pairwise.complete.obs'.}

\item{method}{The method to use.}
}
\value{
Return a real number in [-1, 1].
}
\description{
This function compute the symbolic correlation
}
\details{
Supported interval-valued methods are:
\itemize{
  \item \code{"centers"}: correlation of interval centers.
  \item \code{"B"}: Billard correlation.
  \item \code{"BD"}: Billard-Diday correlation.
  \item \code{"BG"}: Bertrand-Goupil correlation.
}
For \code{"B"}, \code{"BD"}, and \code{"BG"}, the denominator uses the
corresponding method-matched standard deviation.
}
\references{
Bertrand, Patrice and Goupil, Francoise (2000). Descriptive Statistics for
Symbolic Data. In Hans-Hermann Bock and Edwin Diday (eds.),
\emph{Analysis of Symbolic Data}, pp. 106--124. Berlin and Heidelberg:
Springer.

Billard, Lynne and Diday, Edwin (2006). \emph{Symbolic Data Analysis:
Conceptual Statistics and Data Mining}. Chichester, UK: John Wiley and Sons.

Billard, Lynne (2008). Sample covariance functions for complex quantitative
data. In \emph{Proceedings of the World IASC Conference}, pp. 157--163,
Yokohama, Japan.

Rodriguez-Rojas, Oldemar (2000). \emph{Classification et modeles lineaires
en analyse des donnees symboliques}. PhD thesis, Universite Paris IX
Dauphine.
}
\keyword{Symbolic}
\keyword{correlation}
