% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggInterval_scatterplot.R
\name{ggInterval_scatterplot}
\alias{ggInterval_scatterplot}
\title{Scatter plot for two continuous interval variables}
\usage{
ggInterval_scatterplot(data = NULL,mapping = aes(NULL),
showLabels = TRUE, labelSize = 3,
labelPosition = "topright", labelNudgeX = 0, labelNudgeY = 0,
checkOverlap = FALSE, ...)
}
\arguments{
\item{data}{A ggInterval object.It can also be either RSDA object or
classical data frame, which will be automatically convert to ggInterval
data.}

\item{mapping}{Set of aesthetic mappings created by aes() or aes_().
If specified and inherit. aes = TRUE (the default),
it is combined with the default mapping at the top level of
the plot. You must supply mapping if there is no plot mapping.}

\item{showLabels}{Logical. If \code{TRUE}, add row labels to the rectangles.}

\item{labelSize}{Numeric text size used when \code{showLabels = TRUE}.}

\item{labelPosition}{Character label position used when
\code{showLabels = TRUE}. One of \code{"topright"}, \code{"topleft"},
\code{"bottomright"}, or \code{"bottomleft"}.}

\item{labelNudgeX}{Numeric horizontal adjustment applied to label positions.}

\item{labelNudgeY}{Numeric vertical adjustment applied to label positions.}

\item{checkOverlap}{Logical. Passed to \code{geom_text()} to suppress
overlapping labels when needed.}

\item{...}{Others in ggplot2.}
}
\value{
Return a ggplot2 object.
}
\description{
Visualize the distribution of two continuous
interval-valued variables with rectangles whose widths and heights
represent the corresponding intervals.
}
\examples{
Subjects <- substr(rownames(facedata), 1, 3)
ggInterval_scatterplot(facedata, aes(x = AD, y = BC))
ggInterval_scatterplot(
  facedata,
  aes(x = AD, y = BC, fill = Subjects),
  showLabels = TRUE,
  labelSize = 2.6,
  labelPosition = "topright",
  labelNudgeX = 0.8,
  labelNudgeY = 0.15,
  checkOverlap = TRUE,
  col = "black"
)
p <- ggInterval_scatterplot(facedata[1:10, ], aes(x = AD, y = BC, alpha = 0.2))
p + scale_fill_manual(
  labels = rownames(facedata)[1:10],
  values = rainbow(10),
  name = "Group"
)
}
